package me.Demon.BlockWars.Game;

import com.yaohun.GameGoal.GameGoal;
import com.yaohun.GameGoal.api.GameGoalAPI;
import me.Demon.BlockWars.Main;
import me.Demon.BlockWars.Manager.MinerOperate;
import me.Demon.BlockWars.Util.BossBarUtil;
import me.Demon.BlockWars.Util.ComputeBlock;
import me.Demon.BlockWars.Util.GameUtil;
import me.Demon.BlockWars.api.BlockWarsAPI;
import org.bukkit.*;
import org.bukkit.block.Block;
import org.bukkit.boss.BarColor;
import org.bukkit.boss.BarStyle;
import org.bukkit.boss.BossBar;
import org.bukkit.entity.Boss;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Firework;
import org.bukkit.entity.Player;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.FireworkMeta;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.scheduler.BukkitTask;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;
import java.util.concurrent.ExecutionException;

public class GameData {

    public static final Material[] BLOCK_TYPES = new Material[] {Material.IRON_BLOCK, Material.GOLD_BLOCK, Material.DIAMOND_BLOCK, Material.EMERALD_BLOCK};
    private Region region;
    private World world;
    private static boolean started;
    private BossBar bossBar1;
    private BossBar bossBar2;
    private BossBar bossBar3;
    private List<BlockRegion> blockRegionList = new ArrayList<>();
    private int totalBlockAmount = 0; // 当前矿坑中的方块数量
    private int completeBlockGoal = 0;
    private List<BukkitTask> tasks = new ArrayList<>();
    private boolean checking; // 挑战成功
    private int countdownTime = 15; // 挑战成功倒计时

    public GameData(){
        countdownTime = Main.configYml.getCountdownTime(); // 设置倒计时时间
        bossBar1 = Bukkit.createBossBar("今日挑战进度", BarColor.WHITE, BarStyle.SEGMENTED_10);
        bossBar2 = Bukkit.createBossBar("填充进度", BarColor.GREEN, BarStyle.SEGMENTED_20);
        bossBar3 = Bukkit.createBossBar("掉炸弹", BarColor.RED, BarStyle.SOLID);
    }

    public void updateBossBar(){
        refreshCompleteBossBar();
        refreshGiveALikeEffect();
    }

    // 刷新今日挑战进度
    public void refreshCompleteBossBar() {
        int a = GameGoalAPI.getNowcompleteAmount(); // 当前已完成挑战次数
        int b = GameGoalAPI.getCompleteAmount(); // 需要完成多少挑战次数
        double percent = (double) a / (double) b;
        if (a < 0) {
            bossBar1.setTitle("§6今日挑战进度: §4" + a + "§f/" + b);
        } else {
            bossBar1.setTitle("§6今日挑战进度: §f" + a + "/" + b);
        }
        BossBarUtil.setBarProgress(bossBar1,percent);
        BossBarUtil.setBarColor(bossBar1,percent);
    }

    public void refreshGiveALikeEffect(){
        int a = this.totalBlockAmount;
        int b = this.completeBlockGoal;
        double percent = (double) a / (double) b;
        // System.out.println("[调试 - 进度] 填充进度: "+a+"   b = "+b);
        String  percentShow = String.format("%.2f", (percent * 100));
        bossBar2.setTitle("§6填充进度: §f"+a+"/"+b+" §9("+percentShow+"%)");
        // System.out.println("[调试 - 进度] 填充进度: "+percent);
        BossBarUtil.setBarProgress(bossBar2,percent);
        BossBarUtil.setBarColor(bossBar2,percent);
    }

    public void startGame(){
        if(isStarted()){return;}
        setStarted(true);
        initializeGameData(); // 初始化游戏参数
        GameUtil.ClearWorldEntity(); // 清理世界所有生物
        for (Player player : Bukkit.getOnlinePlayers()) {
            initializePlayerData(player); // 初始化玩家数据
        }
        tasks.add(new BukkitRunnable() {
            @Override
            public void run() {
                try {
                    totalBlockAmount = ComputeBlock.getCompleteBlockAmount(blockRegionList).get(); // 每秒刷新一次矿区方块数量
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }.runTaskTimerAsynchronously(Main.plugin, 0L, 20L));
        // 初始化主播参数
        tasks.add(new BukkitRunnable() {
            @Override
            public void run() {
                updateBossBar();
                if (!checking) {
                    checkComplete(); // 检测玩家当前挑战完成进度
                }
            }
        }.runTaskTimer(Main.plugin, 0L, 20L));
    }
    public void initializeGameData(){
        String worldName = "world";
        this.world = Bukkit.getWorld(worldName);
        if(world != null) {
            world.setGameRule(GameRule.DO_DAYLIGHT_CYCLE, false);
            world.setGameRule(GameRule.DO_MOB_LOOT, false);
            world.setGameRule(GameRule.DO_MOB_SPAWNING, false);
            world.setGameRule(GameRule.DISABLE_ELYTRA_MOVEMENT_CHECK, true);
            world.setTime(6000);
        }
        GameGoalAPI.setNowCompleteAmount(0); // 初始化游戏进度
        this.checking = false;
        GameUtil.loadSchematics("miner_11x11"); // 初始化矿区模板
        MinerOperate.initializeMinerSize(); // 初始化矿区大小
        updateBossBar();
        refreshBlockRgionList(16); // 拆分矿区
    }
    /*
    * 初始化主播游戏参数: 生命值、物品栏、BossBar、当前位置
    * */
    public void initializePlayerData(Player player){
        player.getInventory().clear();
        Inventory inv = player.getInventory();
        for (int i = 0;i < 27;i++){
            inv.setItem(i,new ItemStack(Material.IRON_BLOCK));
        }
        player.teleport(region.getHub());
        // 添加boss血条
        BossBarUtil.addBossBar(player,new ArrayList<>(Arrays.asList(bossBar1,bossBar2)));
    }

    private int countdown; // 当前倒计时

    public void checkComplete() {
        if (checking) {
            return;
        }
        if (this.totalBlockAmount >= completeBlockGoal) {
            checking = true;
            countdown = 0;
            tasks.add(new BukkitRunnable() {
                @Override
                public void run() {
                    if (totalBlockAmount < completeBlockGoal) {
                        cancel();
                        stopBukkitTaskList(); // 清理TaskList 并关闭
                        checking = false;
                        Bukkit.broadcastMessage("§c[系统]§a弹坑出现缺口,需要进行填充.");
                        for (Player player : Bukkit.getOnlinePlayers()) {
                            player.sendTitle("§c哎呀!", "§6继续加油!");
                            player.playSound(player.getLocation(), "niganma", 1.0F, 1.0F);
                        }
                        return;
                    }
                    if (countdown >= countdownTime) {
                        BukkitTask task = new BukkitRunnable() {
                            private int i = 0;

                            @Override
                            public void run() {
                                if (i >= 100) {
                                    cancel();
                                    return;
                                }
                                region.getWorld().setTime(region.getWorld().getTime() + 480L);
                                i++;
                            }
                        }.runTaskTimer(Main.plugin, 0L, 1L);
                        tasks.add(task);
                        GameGoalAPI.addNowCompleteAmount(1);
                        stopBukkitTaskList(); // 清理TaskList 并关闭
                        checking = false;
                        Bukkit.broadcastMessage("§c[系统]§a挑战成功! §b游戏进度+1");
                        for (Player player : Bukkit.getOnlinePlayers()) {
                            player.playSound(player.getLocation(), "duolaameng", 1.0F, 1.0F);
                            player.playSound(player.getLocation(), Sound.ENTITY_PLAYER_LEVELUP, 1.0f, 1.1f);
                        }
                        MinerOperate.clearMinerAll(blockRegionList); // 清理矿区中所有方块
                        return;
                    }
                    countdown++;
                    int daojishi = Main.configYml.getCountdownTime();
                    for (Player player : Bukkit.getOnlinePlayers()) {
                        player.sendTitle("§6§l" + (daojishi - countdown), "§6即将挑战成功!");
                    }
                    for (Player player : Bukkit.getOnlinePlayers()) {
                        player.playSound(player.getLocation(), "daojishi", 1.0f, 1.0f);
                    }
                }
            }.runTaskTimer(Main.plugin, 0L, 20L));
            Bukkit.broadcastMessage("§c[系统]§a若§6" + countdownTime + "秒§a内弹坑未出现缺口则挑战成功！");
        }
    }

    /*
    * 清理 BukkitTask 缓存
    * */
    public void stopBukkitTaskList(){
        for (BukkitTask runnable : tasks){
            runnable.cancel();
        }
        tasks.clear();
    }


    /*
     * 当游戏矿区发生变化时需要调用一次这个事件!!!
     * */
    public void refreshBlockRgionList(int chunkSize){
        BlockRegion originalRegion = new BlockRegion((int) region.getMin().getX(), (int) region.getMax().getX());
        List<BlockRegion> blockRegions = new ArrayList<>();
        int min = (int) originalRegion.getMinX();
        int max = (int) originalRegion.getMaxX();
        if(max - min <= chunkSize){
            blockRegions.add(originalRegion);
        }else{
            for (int x = min; x < max; x += chunkSize) {
                int newMax = Math.min(x + chunkSize, max);
                blockRegions.add(new BlockRegion(x, newMax));
            }
        }
        this.blockRegionList = blockRegions;
    }

    public void updateBlockType(Block block) {
        if (!region.isInRegion(block.getLocation())) {
            return;
        }
        int y = block.getY();
        int maxY = (int) region.getMax().getY();// 获取最高y
        int minY = (int) region.getMin().getY();// 获取最低y
        int range = maxY - minY; // 总范围
        double partSize = (double) range / 4; // 每个部分的大小
        int level = 3; // 初始 level
        if (y < minY || y > maxY) {
            level = 0;
        } else {
            // 计算 y 在哪个部分
            if (y < minY + partSize) {
                level = 0;
            } else if (y < minY + 2 * partSize) {
                level = 1;
            } else if (y < minY + 3 * partSize) {
                level = 2;
            }
        }
        Material material = BLOCK_TYPES[level];
        block.setType(material);
    }
    public Material updateBlockType(int y) {
        int maxY = (int) region.getMax().getY();// 获取最高y
        int minY = (int) region.getMin().getY();// 获取最低y
        int range = maxY - minY; // 总范围
        double partSize = (double) range / 4; // 每个部分的大小
        int level = 0; // 初始 level
        if (y < minY || y > maxY) {
            level = 0;
        } else {
            if (y < minY + 2 * partSize) {
                level = 1;
            } else if (y < minY + 3 * partSize) {
                level = 2;
            } else {
                level = 3;
            }
        }
        return BLOCK_TYPES[level];
    }

    public List<BlockRegion> getBlockRegionList() {
        return blockRegionList;
    }
    public Region getRegion() {
        return region;
    }

    public void setRegion(Region region) {
        this.region = region;
    }

    public World getWorld() {
        return world;
    }

    public boolean isStarted() {
        return started;
    }

    public void setStarted(boolean started) {
        GameData.started = started;
    }

    public void setCompleteBlockGoal(int completeBlockGoal) {
        this.completeBlockGoal = completeBlockGoal;
    }
}
