package me.Demon.BlockWars.Util;

import com.sk89q.worldedit.EditSession;
import com.sk89q.worldedit.WorldEdit;
import com.sk89q.worldedit.bukkit.BukkitAdapter;
import com.sk89q.worldedit.math.BlockVector3;
import com.sk89q.worldedit.regions.CuboidRegion;
import me.Demon.BlockWars.Game.BlockRegion;
import me.Demon.BlockWars.Game.Region;
import me.Demon.BlockWars.Main;
import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.block.Block;

import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.atomic.AtomicInteger;

public class ComputeBlock {

    public static CompletableFuture<Integer> getCompleteBlockAmount(List<BlockRegion> blockRegionList) {
        CompletableFuture completableFuture = CompletableFuture.supplyAsync(()->{
            int amount = 0;
            for (BlockRegion blockRegion : blockRegionList) {
                int minX = blockRegion.getMinX();
                int maxX = blockRegion.getMaxX();
                CuboidRegion cuboidRegion = getCuboidRegion(minX, maxX);
                amount += countBlocks(cuboidRegion);
            }
            return amount;
        });
        return completableFuture;
    }

    public static int countBlocks(CuboidRegion cuboidRegion) {
        World world = Main.gameData.getWorld();
        int blockCount = 0;
        for (int y = cuboidRegion.getMinimumPoint().getBlockY(); y <= cuboidRegion.getMaximumPoint().getBlockY(); y++) {
            for (int x = cuboidRegion.getMinimumPoint().getBlockX(); x <= cuboidRegion.getMaximumPoint().getBlockX(); x++) {
                for (int z = cuboidRegion.getMinimumPoint().getBlockZ(); z <= cuboidRegion.getMaximumPoint().getBlockZ(); z++) {
                    Block block = world.getBlockAt(x, y, z);
                    if (!block.getType().equals(Material.AIR)) {
                        blockCount++;
                    }
                }
            }
        }
        return blockCount;
    }

    public static CuboidRegion getCuboidRegion(int minX, int maxX){
        Region region = Main.gameData.getRegion();
        BlockVector3 vector1 = BlockVector3.at(minX, region.getMin().getY(), region.getMin().getZ());
        BlockVector3 vector2 = BlockVector3.at(maxX, region.getMax().getY(), region.getMax().getZ());
        return new CuboidRegion(BukkitAdapter.adapt(region.getWorld()), vector1, vector2);
    }

    /*
    * 每次矿区大小发生变化时需要调用一次!!!!
    * */
    public static int getCompleteBlockGoal(Region region){
        int x = (int) Math.floor(region.getMax().getX()) - (int) Math.floor(region.getMin().getX()) + 1;
        int y = (int) Math.floor(region.getMax().getY()) - (int) Math.floor(region.getMin().getY()) + 1;
        int z = (int) Math.floor(region.getMax().getZ()) - (int) Math.floor(region.getMin().getZ()) + 1;
        return x * y * z;
    }
}
